==> tipp10_3.3.0.orig.tar.bz2: tipp10-v3.3.0/sql/connection.h <==
/*
Copyright (c) 2006-2009, Tom Thielicke IT Solutions

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

/****************************************************************
**
** File name: connection.h
**
****************************************************************/

#ifndef CONNECTION_H
#define CONNECTION_H

#include <QCoreApplication>
#include <QDir>
#include <QFile>
#include <QList>
#include <QMessageBox>
#include <QSettings>
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QStandardPaths>
#include <QString>
#include <QStringList>
#include <QTextStream>

#include "def/defines.h"
#include "def/errordefines.h"
#include "sql/startsql.h"
#include "widget/errormessage.h"

// Database connection to SQLite
static bool createConnection()
{
    // Path do the database
    QString dbPath;
    // Filename of the template database
    QString dbNameTemplate = APP_DB;
    // Filename of the user database
    QString dbNameUser = APP_USER_DB;
    // Connection to SQLite
    QSqlDatabase db;
    QList<QString> lessonId;
    QList<QString> lessonLesson;
    QList<QString> lessonTime;
    QList<QString> lessonToken;
    QList<QString> lessonStrokes;
    QList<QString> lessonErrors;
    QList<QString> lessonStamp;
    QList<QString> charUnicode;
    QList<QString> charTarget;
    QList<QString> charMistake;
    QList<QString> charOccur;

#if APP_PORTABLE
    dbPath = QCoreApplication::applicationDirPath() + "/portable/" + dbNameUser;
#else
    QSettings settings;
    settings.beginGroup("database");
    dbPath = settings.value("pathpro", "").toString();
    settings.endGroup();

    auto newDir
        = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
    QString newLocation = newDir + "/" + dbNameUser;

    if (QFile::exists(dbPath)) {
        QDir().mkpath(newDir);
        QFile(dbPath).rename(newLocation);
        settings.beginGroup("database");
        settings.remove("pathpro");
        settings.endGroup();
    }
    dbPath = newLocation;
#endif

    // User path specified
    if (!QFile::exists(dbPath)) {
        // Try to create new databae in user path
        // Exist a database in the program dir?
#if APP_PORTABLE
        QString dbTemplatePath
            = QCoreApplication::applicationDirPath() + dbNameTemplate;
#else
        QString dbTemplatePath = TIPP10_ROOTDIR + dbNameTemplate;
#endif

        if (QFile::exists(dbTemplatePath)) {
            // A database exist in the program dir
            // -> copy database to appdata location
            QFile file(dbTemplatePath);

            QDir().mkpath(newDir);
            if (file.copy(dbPath)) {
                QFile::setPermissions(
                    dbPath, QFile::permissions(dbPath) | QFile::WriteUser);
            } else {
                ErrorMessage* errorMessage = new ErrorMessage();
                errorMessage->showMessage(ERR_SQL_DB_USER_COPY,
                    ErrorMessage::Type::Warning, ErrorMessage::Cancel::No,
                    QObject::tr("Affected directory:\n") + dbPath);
                return false;
            }
        } else {
            // No database found in program dir
            ErrorMessage* errorMessage = new ErrorMessage();
            errorMessage->showMessage(ERR_SQL_DB_APP_EXIST,
                ErrorMessage::Type::Critical, ErrorMessage::Cancel::Program,
                QObject::tr("Affected directory:\n") + dbPath);
            return false;
        }
    }

    if (QSqlDatabase::contains()) {
        db = QSqlDatabase::database();
        db.close();
    } else {
        db = QSqlDatabase::addDatabase("QSQLITE");
    }

    // Set database
    db.setDatabaseName(dbPath);
    // Open the database
    if (!db.open()) {
        // Error message
        ErrorMessage* errorMessage = new ErrorMessage();
        errorMessage->showMessage(ERR_SQL_CONNECTION,
            ErrorMessage::Type::Critical, ErrorMessage::Cancel::Program,
            QObject::tr("Affected directory:\n") + dbPath);
        return false;
    }

    QSqlQuery queryUpdate;

    if (!queryUpdate.exec("SELECT * FROM db_version ORDER BY version DESC;")) {
        // Error message
        ErrorMessage* errorMessage = new ErrorMessage();
        errorMessage->showMessage(ERR_DB_VERSION_READABLE,
            ErrorMessage::Type::Critical, ErrorMessage::Cancel::Update);
    } else {
        if (!queryUpdate.first()) {
            // Error message
            ErrorMessage* errorMessage = new ErrorMessage();
            errorMessage->showMessage(ERR_DB_VERSION_READABLE,
                ErrorMessage::Type::Critical, ErrorMessage::Cancel::Update);
        } else {
            // Server DB version is 0
            // -> software is too old to update
            if (queryUpdate.value(0).toInt() < (int)COMPILED_UPDATE_VERSION) {

                // Update the database
                QStringList updateFileName = QString(UPDATE_URL_SQL).split("/");

                QFile sqlFile(
                    ":/update/" + updateFileName[updateFileName.count() - 1]);
                if (!sqlFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
                    // Error message
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(ERR_TEMP_FILE_CREATION,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update);
                }

                // Go to the beginning of the version file
                sqlFile.seek(0);

                QTextStream in(&sqlFile);

                QString line;

                // Execute all sql command of the downloaded file
                while (!in.atEnd()) {
                    line = in.readLine();
                    line = line.trimmed();
                    // Exclude comments and empty lines
                    if (line != ""
                        && !line.startsWith("//", Qt::CaseSensitive)) {
                        // Without error handling, because DROP-Statements are
                        // allowed to be invalid (there exist also a IF EXISTS
                        // statement in the SQLite library which suppresses an
                        // error, but it didn't work when I try it)
                        if (!queryUpdate.exec(line)
                            && !line.startsWith("drop", Qt::CaseInsensitive)) {
                            // Error message + failed sql string
                            ErrorMessage* errorMessage = new ErrorMessage();
                            errorMessage->showMessage(ERR_UPDATE_SQL_EXECUTION,
                                ErrorMessage::Type::Critical,
                                ErrorMessage::Cancel::Update, line);
                            break;
                        }
                    }
                }
                StartSql* lessonSql = new StartSql();
                if (!lessonSql->analyzeLessons("lesson")) {
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(ERR_ANALYZE_TABLE_FILL,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update, line);
                }
                if (!lessonSql->analyzeLessons("open")) {
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(ERR_ANALYZE_TABLE_FILL,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update, line);
                }
                if (!lessonSql->analyzeLessons("own")) {
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(ERR_ANALYZE_TABLE_FILL,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update, line);
                }
            }
        }
    }

    return true;
}

#endif // CONNECTION_H

==> tipp10_3.3.1.orig.tar.bz2: tipp10-v3.3.1/sql/connection.h <==
/*
Copyright (c) 2006-2009, Tom Thielicke IT Solutions

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

/****************************************************************
**
** File name: connection.h
**
****************************************************************/

#ifndef CONNECTION_H
#define CONNECTION_H

#include <QCoreApplication>
#include <QDir>
#include <QFile>
#include <QList>
#include <QMessageBox>
#include <QSettings>
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QStandardPaths>
#include <QString>
#include <QStringList>
#include <QTextStream>

#include "def/defines.h"
#include "sql/startsql.h"
#include "widget/errormessage.h"

// Database connection to SQLite
static bool createConnection()
{
    // Path do the database
    QString dbPath;
    // Filename of the template database
    QString dbNameTemplate = t10::app_db;
    // Filename of the user database
    QString dbNameUser = t10::app_user_db;
    // Connection to SQLite
    QSqlDatabase db;
    QList<QString> lessonId;
    QList<QString> lessonLesson;
    QList<QString> lessonTime;
    QList<QString> lessonToken;
    QList<QString> lessonStrokes;
    QList<QString> lessonErrors;
    QList<QString> lessonStamp;
    QList<QString> charUnicode;
    QList<QString> charTarget;
    QList<QString> charMistake;
    QList<QString> charOccur;

#if APP_PORTABLE
    dbPath = QCoreApplication::applicationDirPath() + "/portable/" + dbNameUser;
#else
    QSettings settings;
    settings.beginGroup("database");
    dbPath = settings.value("pathpro", "").toString();
    settings.endGroup();

    auto newDir
        = QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
    QString newLocation = newDir + "/" + dbNameUser;

    if (QFile::exists(dbPath)) {
        QDir().mkpath(newDir);
        QFile(dbPath).rename(newLocation);
        settings.beginGroup("database");
        settings.remove("pathpro");
        settings.endGroup();
    }
    dbPath = newLocation;
#endif

    // User path specified
    if (!QFile::exists(dbPath)) {
        // Try to create new databae in user path
        // Exist a database in the program dir?
        QString dbTemplatePath = t10::tipp10_datadir + dbNameTemplate;

        if (QFile::exists(dbTemplatePath)) {
            // A database exist in the program dir
            // -> copy database to appdata location
            QFile file(dbTemplatePath);

            QDir().mkpath(newDir);
            if (file.copy(dbPath)) {
                QFile::setPermissions(
                    dbPath, QFile::permissions(dbPath) | QFile::WriteUser);
            } else {
                ErrorMessage* errorMessage = new ErrorMessage();
                errorMessage->showMessage(Error::sql_db_user_copy,
                    ErrorMessage::Type::Warning, ErrorMessage::Cancel::No,
                    QObject::tr("Affected directory:\n") + dbPath);
                return false;
            }
        } else {
            // No database found in program dir
            ErrorMessage* errorMessage = new ErrorMessage();
            errorMessage->showMessage(Error::sql_db_app_exist,
                ErrorMessage::Type::Critical, ErrorMessage::Cancel::Program,
                QObject::tr("Affected directory:\n") + dbPath);
            return false;
        }
    }

    if (QSqlDatabase::contains()) {
        db = QSqlDatabase::database();
        db.close();
    } else {
        db = QSqlDatabase::addDatabase("QSQLITE");
    }

    // Set database
    db.setDatabaseName(dbPath);
    // Open the database
    if (!db.open()) {
        // Error message
        ErrorMessage* errorMessage = new ErrorMessage();
        errorMessage->showMessage(Error::sql_connection,
            ErrorMessage::Type::Critical, ErrorMessage::Cancel::Program,
            QObject::tr("Affected directory:\n") + dbPath);
        return false;
    }

    QSqlQuery queryUpdate;

    if (!queryUpdate.exec("SELECT * FROM db_version ORDER BY version DESC;")) {
        // Error message
        ErrorMessage* errorMessage = new ErrorMessage();
        errorMessage->showMessage(Error::db_version_readable,
            ErrorMessage::Type::Critical, ErrorMessage::Cancel::Update);
    } else {
        if (!queryUpdate.first()) {
            // Error message
            ErrorMessage* errorMessage = new ErrorMessage();
            errorMessage->showMessage(Error::db_version_readable,
                ErrorMessage::Type::Critical, ErrorMessage::Cancel::Update);
        } else {
            // Server DB version is 0
            // -> software is too old to update
            if (queryUpdate.value(0).toInt() < t10::compiled_update_version) {

                // Update the database
                QStringList updateFileName = t10::update_url_sql.split("/");

                QFile sqlFile(
                    ":/update/" + updateFileName[updateFileName.count() - 1]);
                if (!sqlFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
                    // Error message
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(Error::temp_file_creation,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update);
                }

                // Go to the beginning of the version file
                sqlFile.seek(0);

                QTextStream in(&sqlFile);

                QString line;

                // Execute all sql command of the downloaded file
                while (!in.atEnd()) {
                    line = in.readLine();
                    line = line.trimmed();
                    // Exclude comments and empty lines
                    if (line != ""
                        && !line.startsWith("//", Qt::CaseSensitive)) {
                        // Without error handling, because DROP-Statements are
                        // allowed to be invalid (there exist also a IF EXISTS
                        // statement in the SQLite library which suppresses an
                        // error, but it didn't work when I try it)
                        if (!queryUpdate.exec(line)
                            && !line.startsWith("drop", Qt::CaseInsensitive)) {
                            // Error message + failed sql string
                            ErrorMessage* errorMessage = new ErrorMessage();
                            errorMessage->showMessage(
                                Error::update_sql_execution,
                                ErrorMessage::Type::Critical,
                                ErrorMessage::Cancel::Update, line);
                            break;
                        }
                    }
                }
                StartSql* lessonSql = new StartSql();
                if (!lessonSql->analyzeLessons("lesson")) {
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(Error::analyze_table_fill,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update, line);
                }
                if (!lessonSql->analyzeLessons("open")) {
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(Error::analyze_table_fill,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update, line);
                }
                if (!lessonSql->analyzeLessons("own")) {
                    ErrorMessage* errorMessage = new ErrorMessage();
                    errorMessage->showMessage(Error::analyze_table_fill,
                        ErrorMessage::Type::Critical,
                        ErrorMessage::Cancel::Update, line);
                }
            }
        }
    }

    return true;
}

#endif // CONNECTION_H
